#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

#include "interface.h"
#include "support.h"

static GtkWidget *dialogbox;
static GtkWidget *logwindow;

void debug(char *format, ...)
{
#ifdef DEBUG
   va_list ap;
   
   va_start(ap,format);
   vfprintf(stderr,format, ap);
   va_end(ap);
#endif
}

void ldebug(int level, char *format, ...)
{
#ifdef DEBUG
   va_list ap;
   
   if (DEBUGLEVEL < level)
     return;
   va_start(ap,format);
   vfprintf(stderr,format, ap);
   va_end(ap);
#endif
}

void defaultAction(GtkButton *button, gpointer user_data)
{
   GtkWidget *dialogbox = 
     GTK_WIDGET(lookup_widget (GTK_WIDGET(button), "dialogbox"));
   if(dialogbox == NULL)
     return;
   gtk_widget_hide(dialogbox);
   gtk_widget_destroy(dialogbox);
}

void showMessageBox(char *text)
{
   GtkWidget *button;
   
   dialogbox = create_dialogbox();
   
   /* Turn on the buttons we want here : */
//  gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogclose")));
//  gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogcancel")));
    gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogok")));
//  gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogyes")));
//  gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogno")));
   
   gtk_widget_show(dialogbox);
   
   if(text != NULL)
     gtk_label_set_text
     (GTK_LABEL(lookup_widget(dialogbox, "dialogtext")), text);
   
   button =lookup_widget (GTK_WIDGET (dialogbox), "dialogok");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (defaultAction), NULL);
}

/* change into showRecordConfirmBox ? */
void showConfirmBox(char *text, gpointer yesAction,
		    gpointer noAction, gpointer rec)
{
   Params *p = getParams();
   GtkWidget *button = NULL;
   GtkCheckMenuItem *confirm_actions;
   void (*ptr2Action)(GtkButton *, gpointer) = yesAction; /* function pointer */
   
   if(yesAction == NULL)
     yesAction = defaultAction;
   if(noAction == NULL)
     noAction = defaultAction;
   
   dialogbox = create_dialogbox();
   
   /* Turn on the buttons we want here : */
// gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogclose")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogcancel")));
// gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogok")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogyes")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (dialogbox,"dialogno")));
   
/* The skip confirmation test must be done AFTER the creation of the dialog_box to 
   avoid the yesAction having a NULL pointer for the button. Deletion of
   the (invisible) dialogbox then takes place in the usual way in the
   yesAction */
   
   confirm_actions = 
     (GTK_CHECK_MENU_ITEM(lookup_widget (p->mainwindow,"settings_confirm_actions")));
   if(confirm_actions != NULL && confirm_actions->active == 0)
   {
      /* do not popup a dialogbox */
      button =lookup_widget (GTK_WIDGET (dialogbox), "dialogyes");
      ptr2Action(GTK_BUTTON(button),rec);
      return;
   }
   
   gtk_widget_show(dialogbox);
   
   if(text != NULL)
     gtk_label_set_text
     (GTK_LABEL(lookup_widget(dialogbox, "dialogtext")), text);
   
   button =lookup_widget (GTK_WIDGET (dialogbox), "dialogcancel");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (defaultAction), NULL);
   
   button =lookup_widget (GTK_WIDGET (dialogbox), "dialogyes");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (yesAction), rec);

   button =lookup_widget (GTK_WIDGET (dialogbox), "dialogno");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
		       GTK_SIGNAL_FUNC (noAction), rec);
}

void showVarArgMessage(char *s,...)
{
   va_list ap;
   char message[MAXMESSAGE];
   
   va_start(ap,s);
   vsnprintf(message,sizeof(message) - 1, s, ap);
   ldebug(2,"Message: %s\n",message);
   
   showMessageBox(message);
}

void showMessage(char *message)
{
   ldebug(2,"Message: %s\n",message);
   
   showMessageBox(message);
}

/* allows for writeLog calls without having to supply the logwindow widget: */
void setLogWindow(GtkWidget *logw)
{
   logwindow = logw;
}

void showSqlError(SQL *sock, char *s,...)
{
   va_list ap;
   char error[MAXMESSAGE];
   
   va_start(ap,s);
   vsnprintf(error,sizeof(error) - 1, s, ap);
   
   writeLog("Error: %s\n",error);
   writeLog("Error: %s\n",sql_error(sock));
}

/* This writes to the logbrowser */
void writeLog(char *s,...)
{
   va_list ap;
   GtkAdjustment *logvbar;
   char logline[MAXMESSAGE];
#ifdef USEGTK2
   GtkTextBuffer *txtbuf;
   GtkTextIter    txtiter;
#endif
   
   va_start(ap,s);
   vsnprintf(logline,sizeof(logline) - 1, s, ap);
   ldebug(2,"%s\n",logline);

   /* TODO: rewrite for GtkTextView (GTK 2.0) */
#ifdef USEGTK2
   txtbuf  = gtk_text_view_get_buffer (GTK_TEXT_VIEW (logwindow));
   gtk_text_buffer_get_end_iter (txtbuf, &txtiter);
   gtk_text_buffer_insert (txtbuf, &txtiter, logline, -1);
   
   /* Always scroll to end of logwindow : */
   gtk_text_buffer_get_end_iter (txtbuf, &txtiter);
   gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (logwindow),
				 &txtiter, 0.0, TRUE, 0.0, 1.0);
#else
   gtk_text_freeze (GTK_TEXT(logwindow));
   gtk_text_insert (GTK_TEXT(logwindow),NULL, NULL, NULL, logline, -1);
 
   /* Clear toplines regularly to avoid heavy memory use */
   if(gtk_text_get_length (GTK_TEXT(logwindow)) > MAXLOGSIZE)
   {
      ldebug(2,"Logsize larger than %d: removing top lines ...\n",MAXLOGSIZE);
      gtk_text_set_point (GTK_TEXT(logwindow),0); /* goto begin of text */
      gtk_text_forward_delete (GTK_TEXT(logwindow), MAXLOGSIZE/4); /* remove 1/4 */
   }

   gtk_text_thaw (GTK_TEXT(logwindow));

   /* Always scroll to end of logwindow : */
   /* Note: this must be AFTER gtk_text_thaw or else we use the old end ! */
   logvbar =  gtk_scrolled_window_get_vadjustment
     (GTK_SCROLLED_WINDOW
      (lookup_widget(logwindow, "logscroller")));
   gtk_adjustment_set_value (logvbar, logvbar->upper);
#endif
}

void saveLogAction(GtkWidget *w, gpointer user_data)
{
   FILE *output;
   Params *p = (Params *) user_data;
   
   ldebug(2,"saveLogAction entered\n");
   
   output = fopen(p->selectedname,"w");
   if(output == NULL)
   {
      showVarArgMessage("Cannot save log:\n open file %s failed !",
		  p->selectedname);
      return;
   }
   
   fprintf(output,gtk_editable_get_chars(GTK_EDITABLE(logwindow),0,-1));
   fclose(output);
}

