#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

#include "interface.h"
#include "support.h"

/* First the callbacks for RecordWindow : */

/* Record menus : */
void on_record_menusave_activate(GtkMenuItem *menuitem, gpointer rec)
{
   Record *r = (Record *)rec;
   newFileSelector("record.out",&(r->selectedname),
			saveRecordAction, NULL, r);
}

void on_record_menuclose_activate(GtkMenuItem *menuitem, gpointer rec)
{
   destroyRecordWindow((Record *)rec);
}

/* Record buttons : */
void on_recordclose_clicked(GtkButton *button, gpointer rec)
{
   destroyRecordWindow((Record *)rec);
}

void on_recordupdate_clicked(GtkButton *button, gpointer rec)
{
   ldebug(2,"on_recordupdate_clicked entered\n");
   
   showConfirmBox("Really update selected record ?\n",
		    GTK_SIGNAL_FUNC (updateRecordAction), NULL, rec);
}

void on_recordcopy_clicked(GtkButton *button, gpointer rec)
{
   newCopyRecordWindow((Record *)rec);
}

void on_recorddelete_clicked(GtkButton *button, gpointer rec)
{
   ldebug(2,"on_recorddelete_clicked entered\n");
   
   showConfirmBox("Really delete selected record ?\n",
		    GTK_SIGNAL_FUNC (deleteRecordAction), NULL, rec);
}

void on_recordrestore_clicked(GtkButton *button, gpointer rec)
{
   int y;
   Record *r = (Record *)rec;
   /* Restore the original field contents of the selected record : */
   for(y=0;y<r->numflds;y++)
      gtk_entry_set_text(r->entries[y], r->contents[y]);
}

void on_recordinsert_clicked(GtkButton *button, gpointer rec)
{
   ldebug(2,"on_recordinsert_clicked entered\n");
   
   showConfirmBox("Really insert record ?\n",
		    GTK_SIGNAL_FUNC (insertRecordAction), NULL, rec);
}

void on_recordsearch_clicked(GtkButton *button, gpointer rec)
{
   ldebug(2,"on_recordsearch_clicked entered\n");
   
   showSearch(getParams(),(Record *)rec);
}

void on_recordclear_clicked(GtkButton *button, gpointer rec)
{
   ;
}

/* Next ensures that every button or menu action has a pointer to the 
   record structure in "user_data". In the glade generated interface.c it
   is impossible to hook the record without modifying create_recordwindow
   which makes updates using glade very complicated !
*/
void connectRecordWindowSignals(Record *rec)
{
   ldebug(2,"connectRecordWindowSignals entered\n");
   ldebug(3,"Record pointer: %ld\n",(long)rec);
   
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"record_menusave")),
		       "activate",GTK_SIGNAL_FUNC (on_record_menusave_activate),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"record_menuclose")),
		       "activate",GTK_SIGNAL_FUNC (on_record_menuclose_activate),
		       (gpointer)rec);
   
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recordclose")),
		       "clicked",GTK_SIGNAL_FUNC (on_recordclose_clicked),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recordupdate")),
		       "clicked",GTK_SIGNAL_FUNC (on_recordupdate_clicked),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recordcopy")),
		       "clicked",GTK_SIGNAL_FUNC (on_recordcopy_clicked),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recorddelete")),
		       "clicked",GTK_SIGNAL_FUNC (on_recorddelete_clicked),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recordrestore")),
		       "clicked",GTK_SIGNAL_FUNC (on_recordrestore_clicked),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recordinsert")),
		       "clicked",GTK_SIGNAL_FUNC (on_recordinsert_clicked),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recordsearch")),
		       "clicked",GTK_SIGNAL_FUNC (on_recordsearch_clicked),
		       (gpointer)rec);
   gtk_signal_connect (GTK_OBJECT(lookup_widget (rec->window,"recordclear")),
		       "clicked",GTK_SIGNAL_FUNC (on_recordclear_clicked),
		       (gpointer)rec);
}

void destroyRecordWindow(Record *rec)
{
   ldebug(2,"destroyRecordwindow entered\n");
   ldebug(3,"Record pointer: %ld\n",(long)rec);

   if(rec->window != NULL) /* delete record doesn't need to have a window ! */
   {
      gtk_widget_hide(rec->window);
      gtk_widget_destroy(rec->window);
   }
   
   destroyRecordstruct(rec);
}

/* Create an empty recordwindow */
void newEmptyWindow(Record *rec, char *title)
{
   int y;
   GtkWidget *frame;
   GtkWidget *label;
   GtkWidget *entry;
   
   ldebug(2,"newEmptyWindow entered\n");
   
   rec->window = create_recordwindow();
   frame       = GTK_WIDGET(lookup_widget (rec->window,"recordframe"));
   label       = GTK_WIDGET(lookup_widget (rec->window,"recordformlabel"));
   gtk_label_set_text(GTK_LABEL(label), title);
   gtk_widget_set_style(label, rec->styles[STLBLDRED]);
   
   rec->gtable = gtk_table_new(rec->numflds, 2, TRUE); /* 2 columns, n rows */

   for(y=0;y<rec->numflds;y++)
   {
      label = gtk_label_new (rec->fields[y].name);
      gtk_widget_ref (label);
      if(y == rec->sortkey)
        gtk_widget_set_style(label, rec->styles[STLBLDRED]);
      else
	gtk_widget_set_style(label, rec->styles[STLBOLD]);
      gtk_widget_show (label);
      gtk_table_attach_defaults (GTK_TABLE (rec->gtable), label, 0,1, y, y+1);
      
      entry = gtk_entry_new ();
      gtk_widget_ref (entry);
      gtk_widget_show (entry);
      gtk_table_attach_defaults (GTK_TABLE (rec->gtable), entry, 1,2,  y, y+1);
      
      rec->entries[y] = GTK_ENTRY(entry);
   }
   
   gtk_table_set_row_spacings (GTK_TABLE (rec->gtable), 10);
   gtk_widget_show(rec->gtable);
   gtk_container_set_border_width (GTK_CONTAINER (frame), 20);
   gtk_container_add (GTK_CONTAINER (frame), rec->gtable);

   connectRecordWindowSignals(rec);
   gtk_widget_hide(rec->window);  /* turn on later when all is finished */
}

void newInsertWindow(Params *p)
{
   Record *rec;
   TabInfo *ti;
   
   ldebug(2,"newInsertWindow entered\n");

   /* Can't create a Record directly from Params :(is there a better way ?) */
   /* -->> YES : make routine to new Record from params directly ;-) */
   if(p->table == NULL)
   {
      showMessage("Please select a table first !");
      return;
   }
   
   ti  = newTabInfostruct(p, RECORDSTAB, p->numflds, p->numrecs);
   rec = newRecordstruct(ti);
   if(rec == NULL)
   {
      showMessage("Memory error:can't create Insert Record window !!");
      return;
   }
   
   newEmptyWindow(rec, "Insert Record");
   
   /* Turn on the buttons we need here : */
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordinsert")));
/* gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordclear"))); */
   gtk_widget_show(rec->window);

   destroyTabInfostruct(ti); /* don't need it anymore */
}

void newModifyWindow(TabInfo *ti)
{
   int y;
   Record *rec;
   
   ldebug(2,"newModifyWindow entered\n");
   
   if(ti == NULL || ti->selectedrow < 0)
   {
      showMessage("Please open browser and/or select a record to modify !");
      return;
   }
   
   rec = newRecordstruct(ti);
   if(rec == NULL)
   {
      showMessage("Memory error:can't create Modify Record window !!");
      return;
   }
   
   newEmptyWindow(rec, "Modify Record");
   
   /* Turn on the buttons we want here : */
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recorddelete")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordupdate")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordcopy")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordrestore")));
   
   /* Copy the field contents of the selected record to entries : */
   for(y=0;y<rec->numflds;y++)
   {
      ldebug(2,"field %2d) %s: %s\n",y,rec->fields[y].name,rec->contents[y]);
      gtk_entry_set_text(rec->entries[y], rec->contents[y]);
   }
   gtk_widget_show(rec->window);
}

/* If clicked on Copy from record window : */
void newCopyRecordWindow(Record *src)
{
   int y;
   Record *rec;
   
   ldebug(2,"newCopyRecordWindow entered\n");
   
   rec = copyRecordstruct(src);
   if(rec == NULL)
   {
      showMessage("Memory error:can't create Copy Record window !!");
      return;
   }
   newEmptyWindow(rec, "Insert Record (copied from other record)");
   
   /* Turn on the buttons we need here : */
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordinsert")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordrestore")));
/* gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordclear")));*/
   
   /* Copy the field entry contents of the source record to destination
    * entries to ensure we always have the latest information : */
   for(y=0;y<rec->numflds;y++)
   {
      ldebug(2,"field %2d) %s: %s\n",y, rec->fields[y].name, rec->contents[y]);
      gtk_entry_set_text(rec->entries[y], rec->contents[y]);
   }
   gtk_widget_show(rec->window);
}

/* If clicked on Copy from mainwindow : */
void newCopyMainWindow(TabInfo *ti)
{
   int y;
   Record *rec;
   
   ldebug(2,"newCopyMainWindow entered\n");

   if(ti == NULL || ti->selectedrow < 0)
   {
      showMessage("Please open browser and/or select a record to copy !");
      return;
   }
   
   rec = newRecordstruct(ti);
   if(rec == NULL)
   {
      showMessage("Memory error:can't create Copy Main window !!");
      return;
   }
   newEmptyWindow(rec,"Insert Record (copied from record list)");
   
   /* Turn on the buttons we want here : */
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordinsert")));
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordrestore")));
/* gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordclear")));*/
   
   /* Copy the field contents of the selected record to entries : */
   for(y=0;y<rec->numflds;y++)
   {
      ldebug(2,"field %2d) %s: %s\n",y, rec->fields[y].name, rec->contents[y]);
      gtk_entry_set_text(rec->entries[y], rec->contents[y]);
   }
   gtk_widget_show(rec->window);
}

void newSearchWindow(Params *p)
{
   Record *rec;
   TabInfo *ti;
   
   ldebug(2,"newSearchWindow entered\n");
   
   if(p->table == NULL)
   {
      showMessage("Please select a table first !");
      return;
   }
   
   ti  = newTabInfostruct(p, SEARCHTAB, p->numflds, p->numrecs);
   rec = newRecordstruct(ti);
   if(rec == NULL)
   {
      showMessage("Memory error:can't create Search Record window !!");
      return;
   }
   
   newEmptyWindow(rec,"Search Records (Use % and/or _ as wildcards)");
   
   /* Turn on the buttons we need here : */
   gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordsearch")));
/* gtk_widget_show(GTK_WIDGET(lookup_widget (rec->window,"recordclear")));*/
   gtk_widget_show(rec->window);

   destroyTabInfostruct(ti); /* don't need it anymore */   
}

/* This should work for all available records */
void saveRecordAction(GtkWidget *w, gpointer user_data)
{
   FILE *output;
   Record *rec = (Record *) user_data;
   int y;
   
   ldebug(2,"saveRecordAction entered\n");
   
   if(rec == NULL)
   {
      showMessage("Cannot save record\n");
      return;
   }
   
   output = fopen(rec->selectedname,"w");
   if(output == NULL)
   {
      showVarArgMessage("Cannot save record:\n open file %s failed !",
			rec->selectedname);
      return;
   }
   
   for(y=0;y<rec->numflds; y++)
   {
      gchar *content = (gchar *)
	gtk_entry_get_text(GTK_ENTRY(rec->entries[y]));
      fprintf(output,"%s: %s\n",rec->fields[y].name,content);
      ldebug(3,"%s: %s\n",rec->fields[y].name,content);
   }
   
   fclose(output);
}
