#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

#include "interface.h"
#include "support.h"

int getSortKey(Params *p)
{
   int y;
   
   ldebug(2,"getSortKey called\n");
   
   /* Find unique primary key */
   for(y=0;y<p->numflds;y++)
   {
      if(IS_PRI_KEY(p->fields[y].flags)&&IS_UNIQUE(p->fields[y].flags))
      {
	 writeLog("Using unique primary key %d for sort in %s\n",y, p->table);
	 return y;
      }
   }
   
   /* Else, find primary key */
   for(y=0;y<p->numflds;y++)
   {
      if(IS_PRI_KEY(p->fields[y].flags))
      {
	 writeLog("Using primary key %d for sort in %s\n",y, p->table);
	 return y;
      }
   }
   
   /* Else, find first unique key */
   for(y=0;y<p->numflds;y++)
   {
      if(IS_UNIQUE(p->fields[y].flags))
      {
	 writeLog("Using unique key %d for sort in %s\n",y, p->table);
	 return y;
      }
   }
   
   writeLog("No unique and/or primary key in %s,"
	    "using first field (0).\n", p->table);
   return 0;
}

/* If we come here, then the user has selected a row in the list. */
void field_selected( GtkWidget      *clist,
		      gint            row,
		      gint            column,
		      GdkEventButton *event,
		      gpointer        data )
{
   TabInfo *ti = (TabInfo *)data;

   ldebug(2,"field_selected entered\n");

   /* This only catches <SPACE> or a doubleclick on the selected record !! : */
   if (event == NULL || event->type == GDK_2BUTTON_PRESS)
   {   
      ldebug(2,"field_selected: doubleclick detected\n");
      
      if(ti->table == NULL)
      {
	 showMessage("Currently no table selected !\n");
	 return;
      }
      writeLog("Setting sort field to %d (%s)\n",
	       row,ti->fields[row].name);
      /* watch out: next destroys current ti struct and forces a refresh !! */
      setSortKey(row);
      return;
   }
   else
      ldebug(3,"You selected field %d: no effect (use doubleclick !)\n",row);
}

gchar *getFieldTypeString(TabInfo *ti, int index)
{
   ldebug(2,"getFieldTypeString entered\n");
   
   switch(ti->fields[index].type)
   {
    case FIELD_TYPE_STRING:     return "String";    break;
    case FIELD_TYPE_VAR_STRING: return "VarString"; break;
    case FIELD_TYPE_FLOAT:      return "Float";     break;
    case FIELD_TYPE_DOUBLE:     return "Double";    break;
    case FIELD_TYPE_LONGLONG:
    case FIELD_TYPE_INT24:
    case FIELD_TYPE_DECIMAL:
    case FIELD_TYPE_TINY:
    case FIELD_TYPE_SHORT:
    case FIELD_TYPE_LONG:       return "Integer";   break;
    case FIELD_TYPE_DATE:       return "Date";      break;
    case FIELD_TYPE_TIME:       return "Time";      break;
    case FIELD_TYPE_TIMESTAMP:  return "TimeStamp"; break;
    case FIELD_TYPE_DATETIME:   return "Datetime";  break;
    case FIELD_TYPE_YEAR:       return "Year";      break;
    case FIELD_TYPE_TINY_BLOB:
    case FIELD_TYPE_MEDIUM_BLOB:
    case FIELD_TYPE_LONG_BLOB:
    case FIELD_TYPE_BLOB:       return "Blob:";     break;
    default:
      writeLog("Unknown fieldtype %d detected !\n",
	       ti->fields[index].type);
      return "Unknown"; break;
   }
}

#define FIELDCOLS 6 /* Number of columns (this is here <> nrflds !!) */
void showFields(Params *p)
{
   gchar length[10];
   int   x;
   TabInfo *ti;
   gchar *row[FIELDCOLS];
   gchar *titles[FIELDCOLS] =
   { "Fieldname", "Tablename", "Length", "Type", "Unique", "Not Null" };
   
   ldebug(2,"showFields entered\n");
   if(p->table == NULL)
   {
      showMessage("Cannot show fields: select a table first !\n");
      return;
   }
   
   destroyTab(p->tabs[FIELDSTAB]);
   /* destroy possible previous FieldTab + TabInfo */
   
   ti = newTabInfostruct(p, FIELDSTAB, FIELDCOLS, p->numflds);
   
   for(x=0;x<ti->nrcols;x++)
     ti->titles[x] = titles[x];
   
   newTab(ti,field_selected,"fieldsclist");
   setTabColumnWidths(ti,FIELDSTAB);
   
   gtk_clist_freeze(GTK_CLIST(ti->clist));
   
   for(x=0; x < ti->nrrows; x++)
   {
      /* convert field length to string: */
      snprintf(length,sizeof(length) - 1,"%8d",p->fields[x].length);
      
      row[0] = ti->fields[x].name;
      row[1] = ti->table;
      row[2] = length;
      row[3] = getFieldTypeString(ti, x);
      row[4] = IS_UNIQUE(ti->fields[x].flags) ? "Y" : "N";
      row[5] = IS_NOT_NULL(ti->fields[x].flags) ? "Y" : "N";
      ldebug(2,"field %d: type %d\n",x,ti->fields[x].type);

      gtk_clist_append(GTK_CLIST(ti->clist),row);
      /* change the sortkey color : */
      if(x == ti->sortkey)
	gtk_clist_set_row_style(GTK_CLIST(ti->clist),
				x, ti->styles[STLRED]);
   }
   
   gtk_clist_thaw(GTK_CLIST(ti->clist));
   gtk_notebook_set_page (GTK_NOTEBOOK(p->notebook), FIELDSTAB);

   p->tabs[FIELDSTAB] = ti;
}

